Encoding: UTF-8
Package: OpenMx
Date: 2026-02-27
Title: Extended Structural Equation Modelling
Authors@R: c(person(c("Steven", "M."), "Boker", role="aut"),
    person(c("Michael", "C."), "Neale", role="aut"), person(c("Hermine", "H."),
    "Maes", role="aut"), person(c("Michael", "J."), "Wilde", role="ctb"), person("Michael",
    "Spiegel", role="aut"), person(c("Timothy", "R."), "Brick", role="aut"),
    person("Ryne", "Estabrook", role="aut"), person(c("Timothy", "C."),
    "Bates", role="aut"), person("Paras", "Mehta", role="ctb"), person("Timo", c("von",
    "Oertzen"), role="ctb"), person(c("Ross", "J."), "Gore", role="aut"),
    person(c("Michael", "D."), "Hunter", role="aut"), person(c("Daniel", "C."), "Hackett", role="ctb"),
    person("Julian", "Karch", role="ctb"), person(c("Andreas", "M."), "Brandmaier", role="ctb"),
    person(c("Joshua", "N."), "Pritikin", role="aut"), 
    person("Mahsa", "Zahery", role="aut"),
    person(c("Robert", "M."), "Kirkpatrick", email="rkirkpatrick2@vcu.edu", role=c("aut","cre")),
    person(c("Yang"), "Wang", role="ctb"),
		person("Ben", "Goodrich", email="goodrich.ben@gmail.com", role="ctb"),
    person(c("Charles"), "Driver", email="driver@mpib-berlin.mpg.de", role="ctb"),
    person(c("Jannik", "H."), "Orzek", email="jannik.orzek@mailbox.org", role="ctb"),
    person(c("Don"), "van den Bergh", email="d.vandenbergh@uva.nl", role="ctb"),
    person("Massachusetts Institute of Technology", role="cph"),
    person("S. G.", "Johnson", role="cph"),
    person("Association for Computing Machinery", role="cph"),
    person("Dieter", "Kraft", role="cph"),
    person("Stefan", "Wilhelm", role="cph"),
    person("Sarah", "Medland", role="cph"),
    person(c("Carl", 'F.'), "Falk", role="cph"),
    person("Matt", "Keller", role="cph"),
    person("Manjunath B G", role="cph"),
    person("The Regents of the University of California", role="cph"),
    person("Lester", "Ingber", role="cph"),
    person("Wong", "Shao Voon", role="cph"),
    person("Juan", "Palacios", role="cph"),
    person("Jiang", "Yang", role="cph"),
		person("Gael", "Guennebaud", role="cph"),
		person("Jitse", "Niesen", role="cph"))
URL: https://openmx.ssri.psu.edu/, https://github.com/OpenMx/OpenMx
BugReports: https://openmx.ssri.psu.edu/forums/
Description: Create structural equation models that can be manipulated programmatically.
    Models may be specified with matrices or paths (LISREL or RAM)
    Example models include confirmatory factor, multiple group, mixture
    distribution, categorical threshold, modern test theory, differential
    Fit functions include full information maximum likelihood, maximum likelihood, and weighted least squares.
    equations, state space, and many others.
	Support and advanced package binaries available at <https://openmx.ssri.psu.edu>.
    The software is described in Neale, Hunter, Pritikin, Zahery, Brick,
    Kirkpatrick, Estabrook, Bates, Maes, & Boker (2016) <doi:10.1007/s11336-014-9435-8>.
SystemRequirements: GNU make, C++17
ByteCompile: yes
Language: en-US
License: Apache License (== 2.0)
LinkingTo: Rcpp, RcppEigen (>= 0.3.3.9.4), RcppParallel, StanHeaders
        (>= 2.10.0.2), BH (>= 1.69.0-1), rpf (>= 0.45), Matrix
Imports: digest, MASS, Matrix (>= 1.2-16), methods, Rcpp, RcppParallel,
        parallel, lifecycle
Depends: R (>= 3.5.0)
Suggests: mvtnorm, numDeriv, roxygen2 (>= 6.1), rpf (>= 0.45),
        snowfall, lme4, covr, testthat, ifaTools, knitr, markdown,
        rmarkdown, reshape2, ggplot2
VignetteBuilder: knitr
LazyLoad: yes
LazyData: yes
Collate: '0ClassUnion.R' 'cache.R' 'MxBaseNamed.R' 'MxData.R'
        'MxDataWLS.R' 'DefinitionVars.R' 'MxReservedNames.R'
        'MxNamespace.R' 'MxSearchReplace.R' 'MxFlatSearchReplace.R'
        'MxUntitled.R' 'MxAlgebraFunctions.R' 'MxExponential.R'
        'MxMatrix.R' 'DiagMatrix.R' 'FullMatrix.R' 'IdenMatrix.R'
        'LowerMatrix.R' 'SdiagMatrix.R' 'StandMatrix.R' 'SymmMatrix.R'
        'UnitMatrix.R' 'ZeroMatrix.R' 'MxMatrixFunctions.R'
        'MxAlgebra.R' 'MxCycleDetection.R' 'MxDependencies.R'
        'MxAlgebraConvert.R' 'MxSquareBracket.R' 'MxEval.R'
        'MxRename.R' 'MxPath.R' 'MxObjectiveMetaData.R'
        'MxExpectation.R' 'MxExpectationNormal.R' 'MxExpectationRAM.R'
        'MxExpectationLISREL.R' 'MxFitFunction.R'
        'MxFitFunctionAlgebra.R' 'MxFitFunctionML.R'
        'MxFitFunctionMultigroup.R' 'MxFitFunctionRow.R'
        'MxFitFunctionWLS.R' 'MxRAMObjective.R' 'MxLISRELObjective.R'
        'MxFIMLObjective.R' 'MxMLObjective.R' 'MxRowObjective.R'
        'MxAlgebraObjective.R' 'MxBounds.R' 'MxConstraint.R'
        'MxInterval.R' 'MxTypes.R' 'MxCompute.R' 'MxModel.R'
        'MxRAMModel.R' 'MxLISRELModel.R' 'MxModelDisplay.R'
        'MxFlatModel.R' 'MxMultiModel.R' 'MxModelFunctions.R'
        'MxModelParameters.R' 'MxUnitTesting.R' 'MxApply.R' 'MxRun.R'
        'MxRunHelperFunctions.R' 'MxSummary.R' 'MxCompare.R'
        'MxSwift.R' 'MxOptions.R' 'MxThreshold.R' 'OriginalMx.R'
        'MxGraph.R' 'MxGraphviz.R' 'MxDeparse.R' 'MxCommunication.R'
        'MxRestore.R' 'MxVersion.R' 'MxPPML.R' 'MxRAMtoML.R'
        'MxErrorHandling.R' 'MxDetectCores.R' 'MxSaturatedModel.R'
        'omxBrownie.R' 'omxConstrainThresholds.R' 'omxGetNPSOL.R'
        'MxFitFunctionR.R' 'MxRObjective.R'
        'MxExpectationHiddenMarkov.R' 'MxExpectationMixture.R'
        'MxExpectationStateSpace.R' 'MxExpectationBA81.R'
        'MxFitFunctionGREML.R' 'MxExpectationGREML.R' 'MxMI.R'
        'MxFactorScores.R' 'MxRobustSE.R' 'MxAvailableOptimizers.R'
        'MxTryHard.R' 'MxSE.R' 'MxAutoStart.R' 'MxRetro.R'
        'MxPenalty.R' 'MxMMI.R' 'omxReadGRMBin.R' 'MxPredict.R' 'zzz.R'
RdMacros: lifecycle
Biarch: true
Version: 2.22.11
RoxygenNote: 7.3.3
NeedsCompilation: yes
Packaged: 2026-03-06 14:21:40 UTC; rmk
Author: Steven M. Boker [aut],
  Michael C. Neale [aut],
  Hermine H. Maes [aut],
  Michael J. Wilde [ctb],
  Michael Spiegel [aut],
  Timothy R. Brick [aut],
  Ryne Estabrook [aut],
  Timothy C. Bates [aut],
  Paras Mehta [ctb],
  Timo von Oertzen [ctb],
  Ross J. Gore [aut],
  Michael D. Hunter [aut],
  Daniel C. Hackett [ctb],
  Julian Karch [ctb],
  Andreas M. Brandmaier [ctb],
  Joshua N. Pritikin [aut],
  Mahsa Zahery [aut],
  Robert M. Kirkpatrick [aut, cre],
  Yang Wang [ctb],
  Ben Goodrich [ctb],
  Charles Driver [ctb],
  Jannik H. Orzek [ctb],
  Don van den Bergh [ctb],
  Massachusetts Institute of Technology [cph],
  S. G. Johnson [cph],
  Association for Computing Machinery [cph],
  Dieter Kraft [cph],
  Stefan Wilhelm [cph],
  Sarah Medland [cph],
  Carl F. Falk [cph],
  Matt Keller [cph],
  Manjunath B G [cph],
  The Regents of the University of California [cph],
  Lester Ingber [cph],
  Wong Shao Voon [cph],
  Juan Palacios [cph],
  Jiang Yang [cph],
  Gael Guennebaud [cph],
  Jitse Niesen [cph]
Maintainer: Robert M. Kirkpatrick <rkirkpatrick2@vcu.edu>
Repository: CRAN
Date/Publication: 2026-03-08 06:10:55 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 02:25:38 UTC; unix
Archs: OpenMx.so.dSYM
