#
#   Copyright 2007-2019 by the individuals mentioned in the source code history
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
# 
#        http://www.apache.org/licenses/LICENSE-2.0
# 
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# nocov start

##' imxGenSwift
##'
##' This is an internal function exported for those people who know
##' what they are doing.
##' @param tc tc
##' @param sites sites
##' @param sfile sfile
imxGenSwift <- function(tc, sites, sfile) {
	cmdline <- paste("swift -tc.file ",tc," -sites.file ",sites," ",sfile)
	system(cmdline)
}
# nocov end
