% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalCharOrNumber-class}
\alias{MxOptionalCharOrNumber-class}
\title{A character, integer, or NULL}
\description{
A character, integer, or NULL
}
