% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalChar-class}
\alias{MxOptionalChar-class}
\title{An optional character}
\description{
An optional character
}
