% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxCompute.R
\docType{class}
\name{MxCompute-class}
\alias{MxCompute-class}
\alias{MxCompute}
\title{MxCompute}
\description{
This is an internal class and should not be used directly.
}
