% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxAlgebraFunctions.R
\name{imxLookupSymbolTable}
\alias{imxLookupSymbolTable}
\title{imxLookupSymbolTable}
\usage{
imxLookupSymbolTable(name)
}
\arguments{
\item{name}{name}
}
\description{
This is an internal function exported for those people who know
what they are doing.
}
