% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\name{MxVersionType-class}
\alias{MxVersionType-class}
\title{A package_version or character}
\description{
A package_version or character
}
