% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxMatrix.R
\name{imxCreateMatrix}
\alias{imxCreateMatrix}
\alias{imxCreateMatrix,MxMatrix-method}
\alias{imxCreateMatrix,DiagMatrix-method}
\alias{imxCreateMatrix,FullMatrix-method}
\alias{imxCreateMatrix,IdenMatrix-method}
\alias{imxCreateMatrix,LowerMatrix-method}
\alias{imxCreateMatrix,SdiagMatrix-method}
\alias{imxCreateMatrix,StandMatrix-method}
\alias{imxCreateMatrix,SymmMatrix-method}
\alias{imxCreateMatrix,UnitMatrix-method}
\alias{imxCreateMatrix,ZeroMatrix-method}
\title{Create a matrix}
\usage{
imxCreateMatrix(
  .Object,
  labels,
  values,
  free,
  lbound,
  ubound,
  nrow,
  ncol,
  byrow,
  name,
  condenseSlots,
  joinKey,
  joinModel
)
}
\arguments{
\item{.Object}{the matrix}

\item{labels}{labels}

\item{values}{values}

\item{free}{free}

\item{lbound}{lbound}

\item{ubound}{ubound}

\item{nrow}{nrow}

\item{ncol}{ncol}

\item{byrow}{byrow}

\item{name}{name}

\item{condenseSlots}{condenseSlots}

\item{joinKey}{joinKey}

\item{joinModel}{joinModel}
}
\description{
This is an internal function exported for those people who know
what they are doing.
}
