% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxCharOrList-class}
\alias{MxCharOrList-class}
\title{A character, list or NULL}
\description{
A character, list or NULL
}
