% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxMultiModel.R
\name{imxFreezeModel}
\alias{imxFreezeModel}
\title{Freeze model}
\usage{
imxFreezeModel(model)
}
\arguments{
\item{model}{model}
}
\description{
Remove free parameters and fit function from model.
}
