% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxExpectation.R
\docType{class}
\name{MxExpectation-class}
\alias{MxExpectation-class}
\alias{MxExpectation}
\title{MxExpectation}
\description{
This is an internal class and should not be used directly.
}
