% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxExponential.R
\name{expm}
\alias{expm}
\alias{omxExponential}
\title{Matrix exponential}
\usage{
expm(x)
}
\arguments{
\item{x}{matrix}
}
\description{
Matrix exponential
}
