% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalInteger-class}
\alias{MxOptionalInteger-class}
\title{An optional integer}
\description{
An optional integer
}
