% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxData.R
\docType{class}
\name{MxDataFrameOrMatrix-class}
\alias{MxDataFrameOrMatrix-class}
\alias{MxDataFrameOrMatrix}
\title{MxDataFrameOrMatrix}
\description{
Internal class that is the union of data.frame and matrix.
}
\details{
Not to be used.
}
