% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalMatrix-class}
\alias{MxOptionalMatrix-class}
\title{An optional matrix}
\description{
An optional matrix
}
