% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalNumeric-class}
\alias{MxOptionalNumeric-class}
\title{An optional numeric}
\description{
An optional numeric
}
