% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{mxSetDefaultOptions}
\alias{mxSetDefaultOptions}
\title{Reset global options to the default}
\usage{
mxSetDefaultOptions()
}
\description{
Reset global options to the default
}
