% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxCharOrLogical-class}
\alias{MxCharOrLogical-class}
\title{A character or logical}
\description{
A character or logical
}
