% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\name{imxHasOpenMP}
\alias{imxHasOpenMP}
\title{imxHasOpenMP}
\usage{
imxHasOpenMP()
}
\description{
This is an internal function exported for those people who know
what they are doing.
}
