% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxListOrNull-class}
\alias{MxListOrNull-class}
\title{An optional list}
\description{
An optional list
}
