% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalLogical-class}
\alias{MxOptionalLogical-class}
\title{An optional logical}
\description{
This is an internal class, the union of NULL and logical.
}
