% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalDataFrameOrMatrix-class}
\alias{MxOptionalDataFrameOrMatrix-class}
\title{An optional data.frame or matrix}
\description{
An optional data.frame or matrix
}
