% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxNamespace.R
\name{imxVerifyReference}
\alias{imxVerifyReference}
\title{imxVerifyReference}
\usage{
imxVerifyReference(reference, stackNumber)
}
\arguments{
\item{reference}{reference}

\item{stackNumber}{stackNumber}
}
\description{
This is an internal function exported for those people who know
what they are doing.
}
