% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxGraph.R
\docType{class}
\name{MxDirectedGraph-class}
\alias{MxDirectedGraph-class}
\alias{MxDirectedGraph}
\title{MxDirectedGraph}
\description{
This is an internal class and should not be used directly.
It is a class for directed graphs.
}
