% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxFitFunction.R
\docType{class}
\name{MxFitFunction-class}
\alias{MxFitFunction-class}
\alias{MxFitFunction}
\title{MxFitFunction}
\description{
This is an internal class and should not be used directly.
}
