% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxOptions.R
\name{imxDetermineDefaultOptimizer}
\alias{imxDetermineDefaultOptimizer}
\title{imxDetermineDefaultOptimizer}
\usage{
imxDetermineDefaultOptimizer()
}
\description{
This is an internal function exported for those people who know
what they are doing.
}
\details{
Returns a character, the default optimizer
}
