% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxCharOrNumber-class}
\alias{MxCharOrNumber-class}
\title{A character or integer}
\description{
A character or integer
}
