% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxExponential.R
\name{logm}
\alias{logm}
\title{Matrix logarithm}
\usage{
logm(x, tol = .Machine$double.eps)
}
\arguments{
\item{x}{matrix}

\item{tol}{tolerance}
}
\description{
Matrix logarithm
}
