% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/0ClassUnion.R
\docType{class}
\name{MxOptionalDataFrame-class}
\alias{MxOptionalDataFrame-class}
\title{An optional data.frame}
\description{
An optional data.frame
}
