% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MxData.R
\name{mxDataDynamic}
\alias{mxDataDynamic}
\alias{MxDataDynamic-class}
\alias{print,MxDataDynamic-method}
\alias{show,MxDataDynamic-method}
\title{Create dynamic data}
\usage{
mxDataDynamic(type, ..., expectation, verbose = 0L)
}
\arguments{
\item{type}{type of data}

\item{...}{Not used.  Forces remaining arguments to be specified by name.}

\item{expectation}{the name of the expectation to provide the data}

\item{verbose}{Increase runtime debugging output}
}
\description{
Create dynamic data
}
